# This file was autogenerated.

=head1 NAME

hydrogen-code.t - unit tests for Hydrogen::Code

=head1 AUTHOR

Toby Inkster E<lt>tobyink@cpan.orgE<gt>.

=head1 COPYRIGHT AND LICENCE

This software is copyright (c) 2022 by Toby Inkster.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=head1 DISCLAIMER OF WARRANTIES

THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

=cut

use 5.008001;
use strict;
use warnings;
use Test2::V0 -target => "Hydrogen::Code";

isa_ok( 'Hydrogen::Code', 'Exporter::Tiny' );

my %EXPORTS = map +( $_ => 1 ), @Hydrogen::Code::EXPORT_OK;

subtest 'execute' => sub {
    ok exists(&Hydrogen::Code::execute), 'function exists';
    ok $EXPORTS{'execute'}, 'function is importable';
};

subtest 'execute_list' => sub {
    ok exists(&Hydrogen::Code::execute_list), 'function exists';
    ok $EXPORTS{'execute_list'}, 'function is importable';
};

subtest 'execute_scalar' => sub {
    ok exists(&Hydrogen::Code::execute_scalar), 'function exists';
    ok $EXPORTS{'execute_scalar'}, 'function is importable';
};

subtest 'execute_void' => sub {
    ok exists(&Hydrogen::Code::execute_void), 'function exists';
    ok $EXPORTS{'execute_void'}, 'function is importable';
};

done_testing; # :)
